import { Injectable } from '@angular/core';
import { throwError } from 'rxjs';
import { catchError } from 'rxjs/operators';
import { AppSettings } from 'src/app/model/config/app-settings';
import { HttpClient } from '@angular/common/http';
import { environment } from 'src/environments/environment';

@Injectable({
  providedIn: 'root',
})
export class ConfigService {
  settings: AppSettings = {
    termsUrl: '',
    privacyURL: '',
    imprintURL: '',
    themes: [
      {
        label: 'Maxlan',
        name: 'maxlan',
      },
    ],
    loadingScreen: {
      background: '',
      logo: '',
    },
    api: {
      url: '',
      wsUrl: '',
      redirector: '',
    },
    keycloak: {
      url: '',
      realm: '',
      clientId: '',
    },
  };

  constructor(private http: HttpClient) {}

  loadSettings(): Promise<void> {
    return new Promise((resolve, reject) => {
      this.http
        .get<AppSettings>(`/assets/config/${environment.config}`)
        .pipe(
          catchError(error => {
            reject(error);
            return throwError(() => error);
          })
        )
        .subscribe(settings => {
          this.settings = settings;
          if (this.settings.themes.length === 0) {
            this.settings.themes.push({
              label: 'Maxlan',
              name: 'maxlan',
            });
          }
          resolve();
        });
    });
  }
}
